/*********************************************//*!@addtogroup file Files*//*@{*/
/*!
 *******************************************************************************
* File identification:      $Id: main.cpp 71 2016-12-31 16:42:30Z cypax $
* Revision of last commit:  $Rev: 71 $
* Author of last commit:    $Author: cypax $
* Date of last commit:      $Date: 2016-12-31 17:42:30 +0100 (Sa, 31 Dez 2016) $
 *******************************************************************************
 *
 * @file          main.cpp
 * @author        Cypax (cypax.net)
 *
 * @brief         Main unit of iCalendar example project. Contains main()
 */
/************************************************************************//*@}*/

/******************************************************************************/
/* Library includes                                                           */
/******************************************************************************/
#include <QtQml>
#include <QDebug>

/******************************************************************************/
/* Own includes                                                               */
/******************************************************************************/
#include "QtQuickControlsApplication.h"
#include "CalendarClient\CalendarManager.h"
#include "CalendarClient\CalendarClient_CalDAV.h"
#include "CalendarClient\CalendarClient_ICS.h"
#include "CalendarClient\CalendarClient.h"

/******************************************************************************/
/* Constants (#define)                                                        */
/***************************************//*!@addtogroup define Constants*//*@{*/
/***** End of: define Constants *****************************************//*@}*/

/******************************************************************************/
/* Public functions                                                           */
/*****************************//*!@addtogroup pubfunct Public functions *//*@{*/

void printUsage()
{
    printf("USAGE:\n"
           "  -swogl  Use software OpenGL (for troubleshooting with graphic errors).\n"
           "  -?      This help text.\n"
           );
}

int main(int argc, char *argv[])
{
  QtQuickControlsApplication app(argc, argv);

  CalendarManager calendarManager(QString(app.applicationDirPath()+"/CalendarManager.ini"));

  if(1 < argc)
  {
      // evaluate parameters
      for(int i=1; i<argc; i++)
      {
          if (0 == strcmp("-?", argv[i]))
          {
              printUsage();
              return 0;
          }
          else if (0 == strcmp("-swogl", argv[i]))
          {
              app.setAttribute(Qt::AA_UseSoftwareOpenGL);
              qDebug() << "Using software OpenGL";
          }
      }
  }

  qmlRegisterUncreatableType<CalendarClient>("CalendarExampleTypes", 1, 0, "CalendarClient", "CalendarClient is uncreatable");
  qmlRegisterUncreatableType<CalendarClient_CalDAV>("CalendarExampleTypes", 1, 0, "CalendarClient_CalDAV", "CalendarClient_CalDAV is uncreatable");
  qmlRegisterUncreatableType<CalendarClient_ICS>("CalendarExampleTypes", 1, 0, "CalendarClient_ICS", "CalendarClient_ICS is uncreatable");
  qmlRegisterUncreatableType<CalendarManager>("CalendarExampleTypes", 1, 0, "CalendarManager", "CalendarManager is uncreatable");

  QQmlApplicationEngine engine;
  engine.rootContext()->setContextProperty("calendarManager", &calendarManager);
  engine.load(QUrl("qrc:///main.qml"));

  return app.exec();
}

/***** End of: pubfunct Public functions ********************************//*@}*/

/**** Last line of source code                                             ****/
